
/** Contains the values and associated display names for all the param data types supported.
 *  This is currently only used for special SQL parameters.
 */
var DATA_TYPES = new Object();
DATA_TYPES['boolean'] = 'Boolean';
//DATA_TYPES['byte'] = 'Byte';
DATA_TYPES['date'] = 'Date';
DATA_TYPES['double'] = 'Double';
DATA_TYPES['float'] = 'Float';
DATA_TYPES['int'] = 'Integer';
//DATA_TYPES['long'] = 'Long';
//DATA_TYPES['short'] = 'Short';
DATA_TYPES['string'] = 'String';
DATA_TYPES['time'] = 'Time';
DATA_TYPES['timestamp'] = 'Timestamp';
DATA_TYPES['xml'] = 'XML';


/**
 * Represents a Parameter for a rule result in a logicsheet document.
 * Provides an interactive editor environment for viewing and modifying the parameter.
 *
 * @param id The unique ID for the param within the result in the current display environment.
 *           This is only used for identifying this param object - it is not stored in the XML at all.
 * @param paramNode The element node of the Parameter in the logicsheet DOM, or null if this is a new param.
 * @param result the Result object that contains this param
 * @param sequence The sequence number for this param if it is a new one.
 * @constructor
 * @author Gerard Smyth
 * @copyright (c) 2005
 * Company Hyfinity Ltd.
 */
function Param(id, paramNode, result, sequence)
{
    this.displayID = id;
    this.result = result;

    this.name="";
    this.type="";
    this.value="";

    /** Stores any errors to show against this param. */
    this.errors = new Object();

    this.useEditArea = false;
    this.editAreaType = '';

    this.paramUse = '';
    this.paramDataType = 'string';

    /*
     * Setup class methods
     */
    if (typeof(__ParamPrototypeCalled) == 'undefined')
    {
        __ParamPrototypeCalled = true;
        Param.prototype.parseXML = parseParamXML;

        Param.prototype.show = showParam;
        Param.prototype.getParamNameDropDown = getParamNameDropDown;
        Param.prototype.getParamValueControl = getParamValueControl;
        Param.prototype.createXPathParamEditor = createXPathParamEditor ;
        Param.prototype.createTextAreaParamEditor = createTextAreaParamEditor;
        Param.prototype.createFileChooserParamEditor = createFileChooserParamEditor;
        Param.prototype.createSelectParamEditor = createSelectParamEditor;
        Param.prototype.createSQLParamEditor = createSQLParamEditor;
        Param.prototype.getParamDataTypeDropDown = getParamDataTypeDropDown

        Param.prototype.attachError = attachParamError;
        Param.prototype.clearErrors = clearParamErrors;
        Param.prototype.hasError = hasParamError;

        Param.prototype.paramNameChanged = paramNameChanged;
        Param.prototype.chooseFile = chooseFile;
        Param.prototype.saveFragment = saveParamFragment;

        Param.prototype.getJavascriptIdentifier = getParamJavascriptIdentifier;
        Param.prototype.getParamXML = getParamXML;

    }

    this.parseXML(paramNode, sequence);
}

/**
 * Parses the paramNode XML fragment to initialise this JavaScript object.
 * @private
 */
function parseParamXML(paramNode, seq)
{
    if (paramNode != null)
    {
        this.name = paramNode.getAttribute("name");

        if ((this.result.operator == 'SQL') && (this.name != 'to_location') && (this.name != 'sql_statement'))
        {
            this.paramUse = 'SQL';
            this.type = 'com.hyfinity.xpath.XPath';
            this.value = paramNode.firstChild.nodeValue;
            this.paramDataType = paramNode.getAttribute("type");
        }
        else
        {
            this.type = paramNode.getAttribute("type");

            if (this.type == 'org.w3c.dom.Node')
            {
                this.value = paramNode.selectSingleNode("*[1]");
            }
            else if (paramNode.firstChild != null)
            {
                this.value = paramNode.firstChild.nodeValue;
            }
        }
    }
    else if (!this.result.rule.ruleBase.isCustom)
    {
        if (seq != null)
        {
            seqNode = this.result.operatorParams.selectSingleNode("hy:param[@sequence = '"+seq+"'][1]");
            this.name = seqNode.getAttribute("name");
            this.type = seqNode.getAttribute("type");
            if (this.type == 'org.w3c.dom.Node')
            {
                this.value = seqNode.selectSingleNode("*[1]");
            }
            else if ((seqNode.getAttribute("editor") == 'Select') && (seqNode.firstChild))
            {
                for (var i = 0; i < seqNode.childNodes.length; ++i)
                {
                    if (seqNode.childNodes.item(i).nodeType == 1)//element node
                    {
                        this.value = seqNode.childNodes.item(i).getAttribute('value');
                        break;
                    }
                }
            }
            else
            {
                this.value = getElementText(seqNode);
            }
        }
    }
}

/**
 * Returns the HTML String to show this parameter
 * <b>Note:</b> The container is assummed to be a table.
 */
function showParam()
{
    paramString = '<tr id="rule_' + this.result.getRule().getID() + '_result_' + this.result.getID() + '_param_' + this.displayID +'_container">';
    paramString += '<td>' + this.getParamNameDropDown() + '</td>';
    paramString += '<td id="rule_' + this.result.getRule().getID() + '_result_' + this.result.getID() + '_param_' + this.displayID +'_value_container">' + this.getParamValueControl() + '</td>';

    if (this.paramUse == 'SQL')
    {
        paramString += '<td>' + this.getParamDataTypeDropDown() + '</td>';
    }

    paramString += '</tr>';
    return paramString;;
}


/**
 * Returns the HTML string to show the param name drop down for this parameter.
 * @return the HTML formatted string containing the select box markup
 */
function getParamNameDropDown()
{
    if (this.result.rule.ruleBase.isCustom)
    {
        return '<span>'+this.name+'</span>';
    }
    else
    {
        var possibleNames = this.result.operatorParams.selectNodes("hy:param[@sequence = ../hy:param[@name = '"+this.name+"' ]/@sequence]");
        if (possibleNames.length == 0)
        {
            return '<span>'+this.name+'</span>';
        }
        else
        {
            var selectString = '<select onchange="'+this.getJavascriptIdentifier()+'.paramNameChanged(this);">';
            for (var i = 0; i < possibleNames.length; ++i)
            {
                var pn = possibleNames.item(i).getAttribute("name");
                var type = possibleNames.item(i).getAttribute("type");
                var dispName = possibleNames.item(i).getAttribute("display_name");
                selectString += "<option value='" + pn + "|" + type + "'";
                if ((pn == this.name) && (type == this.type))
                    selectString += " selected='selected'";
                selectString += ">";
                if ((dispName != null) && (dispName != ''))
                {
                    selectString += dispName;
                }
                else
                {
                    selectString += pn;
                }
                selectString += "</option>";
            }
            selectString += "</select>";
            return selectString;
        }
    }
}

/**
 * Called when the value in the param name dropdown is changed, to update the value control.
 */
function paramNameChanged(dropdown)
{
    var sepChar = dropdown.value.indexOf('|');

    this.name = dropdown.value.substring(0, sepChar);
    this.type = dropdown.value.substring(sepChar+1);
    var op = this.result.operatorParams.selectSingleNode("hy:param[(@name = '"+this.name+"') and (@type = '"+this.type+"')]");


    //set the value to the default for the new param type
    if (this.type == 'org.w3c.dom.Node')
    {
        this.value = op.selectSingleNode("*[1]");
    }
    else if (op.firstChild != null)
    {
        this.value = op.firstChild.nodeValue;
    }
    else
    {
        this.value = '';
    }

    //if the previous param type used an edit area, destroy it
    if (this.useEditArea)
    {
        editAreaLoader.delete_instance('rule_' + this.result.getRule().getID() + '_result_' + this.result.getID() + '_param_' + this.displayID +'_value');
    }

    var cont = document.getElementById("rule_" + this.result.getRule().getID() + "_result_" + this.result.getID() + "_param_" + this.displayID +"_value_container");
    cont.innerHTML = this.getParamValueControl();

    //if the new param type should use an editarea, then create one
    if (this.useEditArea)
    {
        convertToEditArea('rule_' + this.result.getRule().getID() + '_result_' + this.result.getID() + '_param_' + this.displayID +'_value', this.editAreaType);
    }

    this.result.getRule().setModified();
}


/**
 * Returns the HTML string for the container to edit this param value.
 * @return The HTML string
 */
function getParamValueControl()
{
    this.useEditArea = false;

    var editorType
    if ((this.result.rule.ruleBase.isCustom) || (this.result.operatorParams.selectSingleNode("hy:param[(@name = '"+this.name+"') and (@type = '"+this.type+"')]") == null))
    {
        switch (this.type)
        {
          case 'com.hyfinity.xpath.XPath' : editorType = "XPath"; break;
          case 'java.io.File'             : editorType = "FileChooser"; break;
          default                         : editorType = "TextArea"; break;
        }
    }
    else
    {
        editorType = this.result.operatorParams.selectSingleNode("hy:param[(@name = '"+this.name+"') and (@type = '"+this.type+"')]").getAttribute("editor");
    }
    switch (editorType)
    {
        case "XPath"        : return this.createXPathParamEditor();
        case "TextArea"     : return this.createTextAreaParamEditor();
        case "FileChooser"  : return this.createFileChooserParamEditor();
        case "Select"       : return this.createSelectParamEditor();
        case "SQL"          : return this.createSQLParamEditor();
    }
}



/**
 * Creates the HTML needed to show an editor for an XPath param value.
 * @return The HTML string.
 * @private
 */
function createXPathParamEditor()
{
    var context = this.result.getRule().getResultContext();

    //escape the XPath for entering into the HTML string.
    //this currently handles single or double quotes

    if (context != null)
        context = escapeStringQuotes(context);

    var htmlString = '<textarea class="xpathBox"';// rows="4" cols="80"';
    htmlString += ' id="rule_' + this.result.getRule().getID() + '_result_' + this.result.getID() + '_param_' + this.displayID +'_value"';
    htmlString += ' onkeyup="validateXPath(\'rule_' + this.result.getRule().getID() + '_result_' + this.result.getID() + '_param_' + this.displayID +'_value\', \''+this.result.rule.ruleBase.getJavascriptIdentifier()+'\'';
    if (context != null)
        htmlString += ", '" + context + "'";
    htmlString += ');"';

    htmlString += ' onmouseup="saveXPath(this';
    if (context != null)
        htmlString += ", '" + context + "'";
    htmlString += ');validateXPath(\'rule_' + this.result.getRule().getID() + '_result_' + this.result.getID() + '_param_' + this.displayID +'_value\', \''+this.result.rule.ruleBase.getJavascriptIdentifier()+'\'';
    if (context != null)
        htmlString += ", '" + context + "'";
    htmlString += ');" ';

    htmlString += ' onfocus="validateXPath(\'rule_' + this.result.getRule().getID() + '_result_' + this.result.getID() + '_param_' + this.displayID +'_value\', \''+this.result.rule.ruleBase.getJavascriptIdentifier()+'\'';
    if (context != null)
        htmlString += ", '" + context + "'";
    htmlString += ');" ';

    htmlString += ' onchange="'+this.getJavascriptIdentifier()+'.value = this.value; '+this.result.getRule().getJavascriptIdentifier()+'.setModified();" >';

    htmlString += this.value;
    htmlString += '</textarea>';

    if (typeof(this.errors.xpath) != 'undefined')
        htmlString += '<span class="errorIcon" onmouseover="showTipMessage();" onmouseout="hideTipMessage();" _tipMessage="'+this.errors.xpath.getMessage()+'">'+ERROR_TAG_CONTENT_STRING+'</span>';

    htmlString += '<span class="xpath_match_container" id="rule_' + this.result.getRule().getID() + '_result_' + this.result.getID() + '_param_' + this.displayID +'_value_match"></span>';

    htmlString += '<div class="error_container" id="rule_' + this.result.getRule().getID() + '_result_' + this.result.getID() + '_param_' + this.displayID +'_value_error"></div>';

    return htmlString;
}


/**
 * Creates the HTML needed to show a TextArea for entering a param value.
 * @return The HTML string.
 * @private
 */
function createTextAreaParamEditor()
{
    var htmlString = '<textarea ';// rows="4" cols="80"';
    htmlString += 'id="rule_' + this.result.getRule().getID() + '_result_' + this.result.getID() + '_param_' + this.displayID +'_value"';

    if (this.type == 'org.w3c.dom.Node')
    {
        htmlString += ' onchange="'+this.getJavascriptIdentifier()+'.saveFragment(this.value); '+this.result.getRule().getJavascriptIdentifier()+'.setModified();"';
        //add validation of the XML as it is being typed
        htmlString += ' onkeyup="validateXMLFragment(\'rule_' + this.result.getRule().getID() + '_result_' + this.result.getID() + '_param_' + this.displayID +'_value\');"'
        //support creating fragments by dragging from the trees
        htmlString += ' onmouseup="saveXMLFragment(this);this.onkeyup()"';
        htmlString += ' class="fragmentBox">';

        htmlString += new XMLSerializer().serializeToString(this.value);
        htmlString += '</textarea>';

        if (typeof(this.errors.fragment) != 'undefined')
            htmlString += '<span class="errorIcon" onmouseover="showTipMessage();" onmouseout="hideTipMessage();" _tipMessage="'+this.errors.fragment.getMessage()+'">'+ERROR_TAG_CONTENT_STRING+'</span>';

        htmlString += '<div class="error_container" id="rule_' + this.result.getRule().getID() + '_result_' + this.result.getID() + '_param_' + this.displayID +'_value_error"></div>';

        this.useEditArea = true;
        this.editAreaType = 'xml';
    }
    else
    {
        htmlString += ' class="textBox" onchange="'+this.getJavascriptIdentifier()+'.value = this.value; '+this.result.getRule().getJavascriptIdentifier()+'.setModified();" >';
        htmlString += this.value;
        htmlString += '</textarea>';
    }


    return htmlString;
}

/**
 * Creates the HTML needed to show an editor for a FileChooser param value.
 * @return The HTML string.
 * @private
 */
function createFileChooserParamEditor()
{
    var htmlString = '<input type="text" size="65"';
    htmlString += 'id="rule_' + this.result.getRule().getID() + '_result_' + this.result.getID() + '_param_' + this.displayID +'_value"';
    htmlString += 'onchange="'+this.getJavascriptIdentifier()+'.value = this.value; '+this.result.getRule().getJavascriptIdentifier()+'.setModified();" value="' + this.value + '"/>';

    if (typeof(this.errors.document) != 'undefined')
        htmlString += '<span class="errorIcon" onmouseover="showTipMessage();" onmouseout="hideTipMessage();" _tipMessage="'+this.errors.document.getMessage()+'">'+ERROR_TAG_CONTENT_STRING+'</span>';

    htmlString += '<div><a href="javascript:'+this.getJavascriptIdentifier()+'.chooseFile()">Choose File</a></div>';
    return htmlString;
}


/**
 * Creates the HTML needed to show an editor for selecting a param value from a list.
 * @return The HTML string.
 * @private
 */
function createSelectParamEditor()
{
    var htmlString = '<select ';
    htmlString += 'id="rule_' + this.result.getRule().getID() + '_result_' + this.result.getID() + '_param_' + this.displayID +'_value" ';
    htmlString += 'onchange="'+this.getJavascriptIdentifier()+'.value = this.value; '+this.result.getRule().getJavascriptIdentifier()+'.setModified();">';

    //htmlString += '<option value="">Please Select</option>';

    var options = this.result.operatorParams.selectNodes("hy:param[@name = '"+this.name+"']/option");
    for (var i = 0; i < options.length; ++i)
    {
        htmlString += '<option value="'+options.item(i).getAttribute('value')+'"';
        if (options.item(i).getAttribute('value') == this.value)
            htmlString += ' selected="selected"';
        htmlString += '>'+getElementText(options.item(i))+'</option>';
    }


    htmlString += '</select>';

    if (typeof(this.errors.document) != 'undefined')
        htmlString += '<span class="errorIcon" onmouseover="showTipMessage();" onmouseout="hideTipMessage();" _tipMessage="'+this.errors.document.getMessage()+'">'+ERROR_TAG_CONTENT_STRING+'</span>';

    return htmlString;
}



/**
 * Creates the HTML needed to show a TextArea for entering a SQL param value.
 * @return The HTML string.
 * @private
 */
function createSQLParamEditor()
{
    var htmlString = '<textarea ';// rows="4" cols="80"';
    htmlString += 'id="rule_' + this.result.getRule().getID() + '_result_' + this.result.getID() + '_param_' + this.displayID +'_value"';

    htmlString += ' class="sqlTextBox" onchange="'+this.getJavascriptIdentifier()+'.value = this.value; '+this.result.getJavascriptIdentifier()+'.sqlParamValueChanged(this.value); '+this.result.getRule().getJavascriptIdentifier()+'.setModified();" >';
    htmlString += this.value;
    htmlString += '</textarea>';

    this.useEditArea = true;
    this.editAreaType = 'sql';

    return htmlString;
}



/**
 * Returns an HTML string containg the code needed to display the data type dropdown control
 * for this parameter.
 */
function getParamDataTypeDropDown()
{
    var controlHTML = '<select onchange="'+ this.getJavascriptIdentifier() + '.paramDataType = this.value; '+this.result.getRule().getJavascriptIdentifier()+'.setModified();">';

    for (dt in DATA_TYPES)
    {
        controlHTML += '<option value="' + dt + '"'
        if (this.paramDataType == dt)
            controlHTML += ' selected="selected"';

        controlHTML += '>' + DATA_TYPES[dt] + '</option>';
    }

    controlHTML += '</select>';

    return controlHTML;
}



// temporary variable used by xstoreViewerCaller method to store reference to current object.
var currentField;

/**
 * Called by clicking on the 'Choose File' button in the FileChoose value editor.
 * Shows the XStoreViewer to upload and pick the required file.
 */
function chooseFile()
{
    currentField = this;
    if (this.result.operator == 'Transform')
    {
        document.xstoreViewerCaller.xml_permissions.value = document.getElementById('xsl_xstore_collections').value;
        document.xstoreViewerCaller.chosen_collection_name.value = document.getElementById('default_xsl_collection').value;
    }
    else if (this.result.operator == 'Validate')
    {
        document.xstoreViewerCaller.xml_permissions.value = document.getElementById('xsd_xstore_collections').value;
        document.xstoreViewerCaller.chosen_collection_name.value = document.getElementById('default_xsd_collection').value;
    }
    else
    {
        document.xstoreViewerCaller.xml_permissions.value = document.getElementById('all_xstore_collections').value;
        document.xstoreViewerCaller.chosen_collection_name.value = document.getElementById('default_message_collection').value;
    }

    if (document.xstoreViewerCaller.chosen_document_name)
        document.xstoreViewerCaller.chosen_document_name.value = this.value;

    document.xstoreViewerCaller.selection_mode.value = 'id';
    window.xstoreViewer = paramXStoreViewerCallback
    window.open("", "xstoreViewerWindow", 'width=1000, height=600, toolbar=yes, menubar=no, directories=no, scrollbars=yes, resizable=yes, status=yes');
    document.xstoreViewerCaller.submit();
}

/**
 * Simple function to provide xstore call back functionality
 */
function paramXStoreViewerCallback(xstoreName, collectionName, documentName, xstoreId, collectionId, documentId) {
    currentField.value = documentName;
    currentField.result.getRule().setModified();
    currentField.result.toggleResultVisibility();
    currentField.result.toggleResultVisibility();
    document.rulemaker.xml_processing.value += '<add_resource><xfact:resource xmlns:xfact="http://www.hyfinity.com/xfactory"><xfact:xstore>' + xstoreName + '</xfact:xstore><xfact:collection>' + collectionName + '</xfact:collection><xfact:document>' + documentName + '</xfact:document></xfact:resource></add_resource>';
}

/**
 * Called when the value in the TextArea editor for a Node param type is changed.
 * This parses the content into XML and sets this as the value variable.
 */
function saveParamFragment(xmlString)
{
    /*var tempDoc = Sarissa.getDomDocument();
    tempDoc.resolveExternals = false;
    tempDoc.validateOnParse = false;
    tempDoc.loadXML(xmlString);*/
    var tempDoc = getXMLDocument(xmlString)
    if(Sarissa.getParseErrorText(tempDoc) == Sarissa.PARSED_OK)
        this.value = tempDoc.documentElement;
}


/**
 * Attaches the given ValidationError object to this param so
 * that it will be displayed the next time this param is reshown.
 * @param error The ValidationError object describing the error
 */
function attachParamError(error)
{
    switch (error.type)
    {
        case ValidationError.INVALID_PARAM_FRAGMENT :
                    this.errors.fragment = error;
                    break;
        case ValidationError.INVALID_PARAM_XPATH    :
                    this.errors.xpath = error;
                    break;
        case ValidationError.INVALID_PARAM_DOCUMENT :
                    this.errors.document = error;
                    break;
        default :   //not an error associated with a param so ignore.
    }
}

/**
 * Clears all errors associated with this param
 */
function clearParamErrors()
{
    this.errors = new Object();
}

/**
 * Checks whetehr there is an error defined for this param
 * @return A boolean value - true if an error is defined, false otherwise.
 */
function hasParamError()
{
    if (getPropertyCount(this.errors) > 0)
        return true;
    else
        return false;
}


/**
 * Returns the javascript identifier for this Param object.
 */
function getParamJavascriptIdentifier()
{
    return this.result.getParamsIdentifier() + "[" + this.displayID + "]";
}


/**
 * creates an XML string to describe this param object
 * @return The XML formatted string that describes this param.
 */
function getParamXML()
{

    var paramXML = '<hy:param xmlns:hy="http://www.hyfinity.com/xengine" ';

    paramXML += 'name="'+this.name+'" ';

    if (this.paramUse == 'SQL')
        paramXML += 'type="'+this.paramDataType+'">';
    else
        paramXML += 'type="'+this.type+'">';

    if (this.type == "java.lang.String")
    {
        if ((this.value != null) && (this.value != ''))
        {
            paramXML += Sarissa.escape(this.value);
        }
    }
    else if (this.type == "org.w3c.dom.Node")
    {
        if (this.value != null)
        {
            paramXML += new XMLSerializer().serializeToString(this.value);
        }
    }
    else
    {
        paramXML += Sarissa.escape(this.value);
    }

    paramXML += '</hy:param>';

    return paramXML;

}




//********************************* Utility functions ***********************************************


// TODO: Should probably move all these functions as they are not specific to a Parameter

var timeoutID= "";


/**
 * Used to validate the content of the given textbox as valid XML.
 * This initialises a timer to check the validation in 0.5 seconds.
 * This allows this method to be called on keypress, but only actually check
 * when the user has taken a break from typing.
 * Any errors are shown in a container with the same ID as the textbox with _error at the end
 * @param textbox The text container that contains the XML to validate.
 */
function validateXMLFragment(textboxID)
{
    if (timeoutID != "")
    {
        window.clearTimeout(timeoutID);
    }
    timeoutID = window.setTimeout('validateXMLImpl("'+textboxID+'")', ERROR_CHECK_DELAY);
}

/**
 * actually performs the XML validation after the time interval has expired.
 * @private
 */
function validateXMLImpl(validateBoxID)
{
    var errorContainer = document.getElementById(validateBoxID + "_error");

    /*var tempDoc = Sarissa.getDomDocument();
    tempDoc.resolveExternals = false;
    tempDoc.validateOnParse = false;
    tempDoc.loadXML(document.getElementById(validateBoxID).value);*/
    var tempDoc = getXMLDocument(document.getElementById(validateBoxID).value);
    if(Sarissa.getParseErrorText(tempDoc) == Sarissa.PARSED_OK)
    {
        errorContainer.style.display = "none";
        errorContainer.innerHTML = "";
    }
    else
    {
        errorContainer.style.display = "block";
        //errorContainer.innerHTML = '<pre>' + Sarissa.escape(Sarissa.getParseErrorText(tempDoc)) + '</pre>';
        var message = Sarissa.escape(Sarissa.getParseErrorText(tempDoc));
        while (message.indexOf('\n') != -1)
        {
            message = message.substring(0, message.indexOf('\n')) + '<br />' + message.substr(message.indexOf('\n') + 1);

        }
        errorContainer.innerHTML = message;
    }
}



/**
 * Used to validate the content of the given textbox as a valid XPath.
 * This initialises a timer to check the validation in 0.5 seconds.
 * This allows this method to be called on keypress, but only actually check
 * when the user has taken a break from typing.
 * Any errors are shown in a container with the same ID as the textbox with _error at the end
 * @param textbox The text container that contains the XPath to validate.
 */
function validateXPath(textboxID, rulebaseName, context)
{
    if (timeoutID != "")
    {
        window.clearTimeout(timeoutID);
    }
    validateBoxID = textboxID;
    if (typeof(context) == 'undefined')
        timeoutID = window.setTimeout('validateXPathImpl("'+textboxID+'", \''+rulebaseName+'\')', ERROR_CHECK_DELAY);
    else
        timeoutID = window.setTimeout('validateXPathImpl("'+textboxID+'", \''+rulebaseName+'\', \''+escapeStringQuotes(context)+'\')', ERROR_CHECK_DELAY);
}

/**
 * Actually performs the XPath validation after the time interval has expired.
 * @private
 */
function validateXPathImpl(validateBoxID, rulebaseName, context)
{
    var rbobj = eval(rulebaseName);

    var errorContainer = document.getElementById(validateBoxID + "_error");

    /*var tempDoc = Sarissa.getDomDocument();
    tempDoc.loadXML(rbobj.getLogicsheetXML(false));*/
    var tempDoc = getXMLDocument(rbobj.getLogicsheetXML(false));
    var xpath = document.getElementById(validateBoxID).value

    if (xpath == "")
    {
        errorContainer.style.display = "block";
        errorContainer.innerHTML = '<pre>XPath Required</pre>';
        //clear any match information
        var matchCont = document.getElementById(validateBoxID + "_match")
        if (matchCont != null)
        {
            matchCont.innerHTML = "";
        }
        if (rbobj.instanceManager != null)
        {
            rbobj.instanceManager.clearInputMatches();
            rbobj.instanceManager.clearOutputMatches();

        }
    }
    else
    {
        //remove sections from the xpaths that the validator will not understand (eg workspaces)
        //If the entered xpath basically defines two (or more) xopaths joined together with
        // 'and' or 'or' commands then these will be returned as seperate xpaths.
        //The first entry in the resulting array wil be the given xpath without workspace definitions,
        //and each subsequent xpath will be one of hte independant xpaths to evaluate
        var strippedXPaths = stripXPath(xpath, rbobj);
        try
        {
            Sarissa.setXpathNamespaces(tempDoc, rbobj.getNamespaceString());
            for (var i = 1; i < strippedXPaths.length; ++i)
            {
                //alert("checking: " + strippedXPaths[i]);
                tempDoc.selectSingleNode(strippedXPaths[i]);
            }

            errorContainer.style.display = "none";
            errorContainer.innerHTML = "";

            //no errors so check for matches
            showXPathMatches(strippedXPaths[0], rulebaseName, validateBoxID + "_match", context);
        }
        catch (e)
        {
            var message = '';
            if (e.description) //IE
                message = e.description;
            else if (e.message) //FF
                message = e.message;

            if ((message.indexOf("Expression must evaluate to a node-set.") == 0) || //MSXML 4
                (message.indexOf("Expression does not return a DOM node.") == 0) || //MSXML 3
                (message.indexOf("The expression cannot be converted to return the specified type") == 0) || //FireFox
                (message.indexOf("TYPE_ERR: DOM XPath Exception 52") == 0)) //Google Chrome
            {
                //this error is because the xpath doesnt give a nodeset - probably using a function
                //like concat or not.  Therefore just ignore the error
                //TODO: This is a bit of a messy way of detecting these errors - possibly look for a better approach
                errorContainer.style.display = "none";
                errorContainer.innerHTML = "";
                return;
            }

            errorContainer.style.display = "block";
            //errorContainer.innerHTML = '<pre>' + e.description + '</pre>';
            while (message.indexOf('\n') != -1)
            {
                message = message.substring(0, message.indexOf('\n')) + '<br />' + message.substr(message.indexOf('\n') + 1);

            }

            errorContainer.innerHTML = message;

            //clear any match information
            var matchCont = document.getElementById(validateBoxID + "_match")
            if (matchCont != null)
            {
                matchCont.innerHTML = "";
            }
            if (rbobj.instanceManager != null)
            {
                rbobj.instanceManager.clearInputMatches();
                rbobj.instanceManager.clearOutputMatches();

            }
        }
    }
}


/**
 * Removes sections from the given xpath that are actually valid, but will be thrown
 * up as errors when trying to evaluate it. (eg workspaces)
 * Also seperates up xpaths composed of two sperate xpaths ANDed or ORed together into
 * their independent components.
 * @param xpath The XPath to manipulate.
 * @param rulebase The RuleBase object to use to get access to workspace definitions etc.
 * @return An array of xpaths.  The first entry in the array will be the given xpath without
 *                              workspace definitions,
 *                              Each subsequent entry will be one of the independant xpaths
 * @private
 */
function stripXPath(xpath, rulebase)
{
    var newXPath = xpath;
    for (ws in rulebase.workspaces)
    {
        var regExp = new RegExp("\\$"+ws+"/", "g");
        newXPath = newXPath.replace(regExp, "/");
        if (newXPath == ("$" + ws))
        {
            newXPath = "";
            break;
        }
    }

    //remove the '$factbase' implicit workspace
    var regExp = new RegExp("\\$factbase/", "g");
    newXPath = newXPath.replace(regExp, "/");

    //QUESTION: Anything else to strip out??????

    var returnArray = new Array();
    returnArray[0] = newXPath;

    return returnArray.concat(splitXPaths(newXPath));
}


function splitXPaths(xpath)
{
    var openBrackets;
    var closeBrackets;
    var xpaths = new Array();

    if ((xpath == null) || (xpath == ''))
    {
        return xpaths;
    }

    var andPos = xpath.indexOf(" and ");

    if (andPos != -1)
    {
        openBrackets = countPreviousCharacter(xpath, '(', andPos);
        closeBrackets = countPreviousCharacter(xpath, ')', andPos);
        openSquareBrackets = countPreviousCharacter(xpath, '[', andPos);
        closeSquareBrackets = countPreviousCharacter(xpath, ']', andPos);
        if ((openBrackets == closeBrackets) && (openSquareBrackets == closeSquareBrackets))
        {
            xpaths = xpaths.concat(splitXPaths(xpath.substr(0, andPos)));

            xpaths = xpaths.concat(splitXPaths(xpath.substr(andPos + 5)));
        }
        else
        {
            xpaths[xpaths.length] = xpath;
        }
    }
    else
    {
        xpaths[xpaths.length] = xpath;
    }

    var finalXPaths = new Array();

    for (var i = 0; i < xpaths.length; ++i)
    {
        var orPos = xpaths[i].indexOf(" or ");

        if (orPos != -1)
        {
            openBrackets = countPreviousCharacter(xpaths[i], '(', orPos);
            closeBrackets = countPreviousCharacter(xpaths[i], ')', orPos);
            openSquareBrackets = countPreviousCharacter(xpaths[i], '[', orPos);
            closeSquareBrackets = countPreviousCharacter(xpaths[i], ']', orPos);
            if ((openBrackets == closeBrackets) && (openSquareBrackets == closeSquareBrackets))
            {
                finalXPaths = finalXPaths.concat(splitXPaths(xpaths[i].substr(0, orPos)));

                finalXPaths = finalXPaths.concat(splitXPaths(xpaths[i].substr(orPos + 5)));
            }
            else
            {
                finalXPaths[finalXPaths.length] = xpaths[i];
            }
        }
        else
        {
            finalXPaths[finalXPaths.length] = xpaths[i];
        }
    }

    return finalXPaths;
}


function countPreviousCharacter(string, character, pos)
{
    if (string.length < pos)
        pos = string.length;

    var count = 0;
    for (var i = 0; i < pos; ++i)
    {
        if (string.charAt(i) == character)
            ++count;
    }

    return count;

}


/**
 * Evaluates the given xpath against both instance documents and
 * indicates any matches in the given container.
 *
 * @param xpath A String containing the XPath to evaluate
 * @param rulebaseName The name (javascript identifier) of the RuleBase object that defines the context in
 *                     which this xpath is being evaluated
 * @param containerID The ID of the HTML container in which to output the match indicators.  This must not contain
 *                    any other information, as it will be cleared out each time this function is called.
 * @param context An optional string containing the XPath to use as the context point for the main xpath
 */
function showXPathMatches(xpath, rulebaseName, containerID, context)
{
    var rulebase = eval(rulebaseName);

    if (typeof(context) != 'undefined')
        context = stripXPath(context, rulebase)[0];

    var container = document.getElementById(containerID);
    //check there is a manager object defined for the instances
    if ((rulebase.instanceManager != null) && (container != null))
    {
        container.innerHTML = "";
        //check the input document
        if (rulebase.instanceManager.inputDoc != null)
        {
            var inputMatches = rulebase.instanceManager.evaluateXPathAgainstInput(xpath, context);
            if ((inputMatches != null) && (inputMatches.length > 0 ))
            {
                //xpath matches against input document
                if (typeof(context) == 'undefined')
                    container.innerHTML += '<a class="input_match" href="javascript:'+rulebase.instanceManager.getJavascriptIdentifier()+'.showMatchesOnInput(\''+escapeStringQuotes(xpath)+'\', true)" title="This XPath matches '+inputMatches.length+' element(s) in the example input document.  Click to highlight the matches.">' + inputMatches.length + '</a>';
                else
                    container.innerHTML += '<a class="input_match" href="javascript:'+rulebase.instanceManager.getJavascriptIdentifier()+'.showMatchesOnInput(\''+escapeStringQuotes(xpath)+'\', true, \''+escapeStringQuotes(context)+'\')" title="This XPath matches '+inputMatches.length+' element(s) in the example input document.  Click to highlight the matches.">' + inputMatches.length + '</a>';
            }
            rulebase.instanceManager.showMatchesOnInput(xpath, false, context);
        }
        //check the output document
        if (rulebase.instanceManager.outputDoc != null)
        {
            var outputMatches = rulebase.instanceManager.evaluateXPathAgainstOutput(xpath, context);
            if ((outputMatches != null) && (outputMatches.length > 0 ))
            {
                //xpath matches against output document
                if (typeof(context) == 'undefined')
                    container.innerHTML += '<a class="output_match" href="javascript:'+rulebase.instanceManager.getJavascriptIdentifier()+'.showMatchesOnOutput(\''+escapeStringQuotes(xpath)+'\', true)" title="This XPath matches '+outputMatches.length+' element(s) in the example output document.  Click to highlight the matches.">' + outputMatches.length + '</a>';
                else
                    container.innerHTML += '<a class="output_match" href="javascript:'+rulebase.instanceManager.getJavascriptIdentifier()+'.showMatchesOnOutput(\''+escapeStringQuotes(xpath)+'\', true, \''+escapeStringQuotes(context)+'\')" title="This XPath matches '+outputMatches.length+' element(s) in the example output document.  Click to highlight the matches.">' + outputMatches.length + '</a>';
            }
            rulebase.instanceManager.showMatchesOnOutput(xpath, false, context);
        }
    }
}

/**
 * Escapes all quote characters (' or ") in the given string
 * so they can be placed in an HTML attribute value correctly.
 * apostraphes are escaped with \ characters, where as quotes are converted to &quot;
 * @param text the String to escape.
 * @return the given string with all quote charcters escaped.
 */
function escapeStringQuotes(text)
{
    return text.replace(/'/gi , "\\'").replace(/"/gi , "&quot;");
}

/**
 * Converts the textarea with the given ID to be an editarea
 */
function convertToEditArea(textareaid, format)
{
    try
    {
        if (format == 'sql')
        {
            editAreaLoader.init({
                id : textareaid
                ,syntax: "sql"          // syntax to be uses for highgliting
                ,start_highlight: true  // to display with highlight mode on start-up
                ,toolbar: "search, go_to_line, |, undo, redo, |, select_font,|, change_smooth_selection, highlight, reset_highlight, word_wrap, |, help"
                ,font_size: 8
                ,replace_tab_by_spaces: 4
                ,change_callback: 'editAreaChangeCallbackFunction'
            });
        }
        else
        {
            editAreaLoader.init({
                id : textareaid
                ,syntax: "xml"          // syntax to be uses for highgliting
                ,start_highlight: true  // to display with highlight mode on start-up
                ,toolbar: "search, go_to_line, |, undo, redo, |, select_font,|, change_smooth_selection, highlight, reset_highlight, word_wrap, pretty_print, |, help"
                ,plugins: "validator, xmlhelper"
                ,font_size: 8
                ,replace_tab_by_spaces: 4
                ,change_callback: 'editAreaChangeCallbackFunction'
            });
        }
    }
    catch (e)
    { } //ignore for now
}


/**
 * Callback function for the editarea on change event.
 * this just takes the current value of the editarea that fired the event,
 * and places it in the standard textarea for the field.
 * We then trigger the onchange event of this field.
 * This should ensure that any onchange events for the field get fired regardless of whether it is rendered as an
 * editarea or not.
 */
function editAreaChangeCallbackFunction(tid)
{
    document.getElementById(tid).value = editAreaLoader.getValue(tid);
    document.getElementById(tid).onchange();
}