/**
 * Represents a Result for a rule in a logicsheet document.
 * Provides an interactive editor environment for viewing and modifying the result.
 *
 * @param id The unique ID for the result within the rule in the current display environment.
 *           This is only used for identifying this result object - it is not stored in the XML at all.
 * @param orderingNum An integer value used to order results - Lower numbers come first
 * @param resultNode The element node of the Result in the logicsheet DOM.
 * @param rule the Rule object that contains this result
 * @constructor
 * @author Gerard Smyth
 * @copyright (c) 2005
 * Company Hyfinity Ltd.
 */
function Result(id, orderingNum, resultNode, rule, state)
{

    this.displayID = id;
    this.resultNode = resultNode;
    this.rule = rule;
    this.order = orderingNum;

    this.name="";
    this.operator="";
    this.disabled = false;
    this.target = new Object();
    this.target.parallel = false;

    //stores all the currently defined Param objects for this result
    this.currentParams = new Object();

    //stores the XML fragment describing the possible parameters for this operator type
    this.operatorParams = null;

    /** Stores whether this result should be shown expanded (ie params visible) */
    this.expanded = false;

    /** Initialise the object that will keep track of all errors with this result.*/
    this.errors = new Object();

    /*
     * Setup class methods
     */
    if (typeof(__ResultPrototypeCalled) == 'undefined')
    {
        __ResultPrototypeCalled = true;
        Result.prototype.parseXML = parseResultXML;

        Result.prototype.show = showResult;
        Result.prototype.showResultHeader = showResultHeader;

        Result.prototype.toggleResultVisibility = toggleResultVisibility;
        Result.prototype.operatorChanged = operatorChanged;

        Result.prototype.sqlParamValueChanged = sqlParamValueChanged;
        Result.prototype.changeDBConnection = changeDBConnection;

        Result.prototype.setDisabled = setResultDisabled;

        Result.prototype.attachError = attachResultError;
        Result.prototype.clearErrors = clearResultErrors;
        Result.prototype.hasError = hasResultError;
        Result.prototype.hasParamError = hasResultParamError;

        Result.prototype.getJavascriptIdentifier = getResultJavascriptIdentifier;
        Result.prototype.getParamsIdentifier = getParamsIdentifier;
        Result.prototype.getRule = getResultRule;
        Result.prototype.getID = getResultID;
        Result.prototype.getResultXML = getResultXML;
        Result.prototype.getStateXML = getResultStateXML;

        Result.prototype.showContextMenu = showResultContextMenu;
    }

    this.parseXML(state);
}

/**
 * Parses the DOM Node for the result, and intialises this result object
 * @private
 */
function parseResultXML(state)
{
    if (this.resultNode != null)
    {
        if ((typeof(state) != 'undefined') && (state != null))
        {
            if (state.getAttribute("visible") == 'true')
                this.expanded = true;
        }

        this.name = this.resultNode.getAttribute("id");
        var disable = this.resultNode.getAttribute("disable");
        if (disable == 'true')
            this.disabled = true;

        var op = this.resultNode.selectSingleNode("hy:operator");
        if ((op != null) && (op.firstChild != null))
        {
            this.operator = op.firstChild.nodeValue;
        }
        var tgt = this.resultNode.selectSingleNode("hy:target");
        if (tgt != null)
        {
           this.target.name = tgt.getAttribute("name");
           this.target.action = tgt.getAttribute("action");
           this.target.update = tgt.getAttribute("update");
           if (tgt.getAttribute("type") != null)
           {
               this.target.type = tgt.getAttribute("type");
           }
           if (tgt.getAttribute("parallel") != null)
           {
               if (tgt.getAttribute("parallel") == 'true')
                   this.target.parallel = true;
           }
        }

        this.operatorParams = resultOperatorsDoc.selectSingleNode("/hy:rulemaker/hy:results/hy:operator[@name = '"+this.operator+"']/hy:params");
        paramsNL = this.resultNode.selectNodes("hy:target/hy:params/hy:param");
        for (var i = 0; i < paramsNL.length; ++i)
        {
           this.currentParams[i] = new Param(i, paramsNL.item(i), this);
        }
    }
}

/**
 * Displays the viewer/editor for this result in the given container
 * @param containerDiv, the container in which to show the result
 * @param num the number of this result in displayed order - will be 1 for first result shown, then 2 etc
 */
function showResult(containerDiv, num)
{
    resultContainer = document.createElement("div");
    resultContainer.setAttribute("id", "rule_" + this.rule.getID() + "_result_" + this.displayID + "_container");
    resultContainer.oncontextmenu = new Function("return "+this.getJavascriptIdentifier()+".showContextMenu(arguments[0])");
    resultContainer.className = "collapsible_section possibleFloat marginAdjustment";
    if (num == 1)
        resultContainer.style.clear = 'none';
    resultContainer.setAttribute("dd_locations_source", "top, bottom, into");
    containerDiv.appendChild(resultContainer);
    this.showResultHeader(resultContainer, num);

    if ((this.expanded == true) || (this.hasParamError()) || (typeof(this.errors.service) != 'undefined') || (typeof(this.errors.dbconnection) != 'undefined'))
        this.toggleResultVisibility();

    this.setDisabled(this.disabled, true);
}

/**
 * Shows the result header information (name and operator) in the given container.
 * This header has the expansion button to show all params.
 * @param containerDiv The HTML container in which to show the header
 * @param num the number of this result in displayed order - will be 1 for first result shown, then 2 etc
 * @private
 */
function showResultHeader(containerDiv, num)
{
    var headingString = '<div id="rule_'+this.rule.getID()+'_result_header_'+this.displayID+'" class="collapsible_heading"';
    if (num == 1)
        headingString += ' style="clear:none;"';
    headingString += '><table border="0"><tr>';
    headingString += '<td rowspan="2"><a id="rule_'+this.rule.getID()+'_result_header_button_'+this.displayID+'" href="javascript:'+this.getJavascriptIdentifier()+'.toggleResultVisibility()" alt="show/hide action details" class="collapsible_hidden"><span>show/hide</span></a></td>';
    headingString += '<td rowspan="2"><label for="rule_'+this.rule.getID()+'_result_'+this.displayID+'_operator">Action:</label></td>';
    headingString += '<td rowspan="2"';
    if (this.rule.ruleBase.isCustom)
        headingString += ' style="padding-right : 10px;"';
    headingString += ">" + getOperatorsSelectHTML("rule_"+this.rule.getID()+"_result_"+this.displayID+"_operator", this.operator, this.rule.ruleBase.isCustom, this.getJavascriptIdentifier()+".operatorChanged(this)");



    if (typeof(this.errors.operator) != 'undefined')
        headingString += '<span class="errorIcon" onmouseover="showTipMessage();" onmouseout="hideTipMessage();" _tipMessage="'+this.errors.operator.getMessage()+'">'+ERROR_TAG_CONTENT_STRING+'</span>';

    if (!this.rule.ruleBase.isCustom)
        headingString += '<span class="hintIcon" id="rule_'+this.rule.getID()+'_result_header_'+this.displayID+'_hint" onmouseover="showTipMessage(\'TR\');" onmouseout="hideTipMessage();" _tipMessage="'+getEscapedActionHint(this.operator)+'">'+ERROR_TAG_CONTENT_STRING+'</span>';

    headingString += '</td>';
    headingString += '<td rowspan="2"><label for="rule_'+this.rule.getID()+'_result_'+this.displayID+'_name">Name:</label></td>';
    //headingString += '<td rowspan="2"><input type="text" size="55" name="rule_'+this.rule.getID()+'_result_'+this.displayID+'_name" id="rule_'+this.rule.getID()+'_result_'+this.displayID+'_name" value="'+this.name+'" onchange="this.value = this.value.replace(/\\s/g, \'\'); '+this.getJavascriptIdentifier()+'.name = this.value; '+this.rule.getJavascriptIdentifier()+'.setModified();'+this.rule.getJavascriptIdentifier()+'.updateRuleSummary();"/>';
    headingString += '<td rowspan="2"><input type="text" size="55" name="rule_'+this.rule.getID()+'_result_'+this.displayID+'_name" id="rule_'+this.rule.getID()+'_result_'+this.displayID+'_name" value="'+this.name+'" onchange="this.value = collapseWhitespace(this.value); '+this.getJavascriptIdentifier()+'.name = this.value; '+this.rule.getJavascriptIdentifier()+'.setModified();'+this.rule.getJavascriptIdentifier()+'.updateRuleSummary();"/>';

    if (typeof(this.errors.name) != 'undefined')
        headingString += '<span class="errorIcon" onmouseover="showTipMessage();" onmouseout="hideTipMessage();" _tipMessage="'+this.errors.name.getMessage()+'">'+ERROR_TAG_CONTENT_STRING+'</span>';

    headingString += '</td>';

    //add in the disabled option
    headingString += '<td rowspan="2"><label for="rule_'+this.rule.getID()+'_result_'+this.displayID+'_disabled">Disabled:</label></td>';
    headingString += '<td rowspan="2"><input type="checkbox" name="rule_'+this.rule.getID()+'_result_'+this.displayID+'_disabled" id="rule_'+this.rule.getID()+'_result_'+this.displayID+'_disabled" onclick="'+this.getJavascriptIdentifier()+'.setDisabled(this.checked);"';
    if (this.disabled)
        headingString += ' checked="checked"'
    headingString += '/></td>';

    if (!this.rule.ruleBase.isCustom)
    {
        headingString += '<td><a class="up_button" href="javascript:'+this.rule.getJavascriptIdentifier()+'.moveResultUp(\''+this.displayID+'\');" title="Move Action Up"><span>Move Action Up</span</a></td>';
        headingString += '<td rowspan="2"><a class="remove_button" href="javascript:'+this.rule.getJavascriptIdentifier()+'.deleteResult(\''+this.displayID+'\');" title="Delete Action"><span>Delete Action</span</a></td>';
        headingString += '</tr><tr>';
        headingString += '<td><a class="down_button" href="javascript:'+this.rule.getJavascriptIdentifier()+'.moveResultDown(\''+this.displayID+'\');" title="Move Action Down"><span>Move Action Down</span</a></td>';
    }

    headingString += '</tr></table>';

    headingString += '</div>';

    containerDiv.innerHTML = headingString;
}


/** Toggles the visibility of this result object (including all params.
 */
function toggleResultVisibility()
{
    var existingParams = document.getElementById("rule_" + this.rule.getID() + "_result_" + this.displayID + "_detail");
    if (existingParams == null)
    {
        var cont = document.createElement("div");
        cont.setAttribute("id", "rule_" + this.rule.getID() + "_result_" + this.displayID + "_detail");
        cont.className = "result_content";

        var paramCount = getPropertyCount(this.currentParams);

        //show parameters
        var paramCont = '<table border="0">';


        if (this.operator == 'SQL')
        {
            paramCont += '<tr style="height : 25px; vertical-align : middle;"><td>Database:</td>';
            paramCont += '<td><span id="rule_' + this.rule.getID() + '_result_' + this.displayID + '_target_name">'
            if ((typeof(this.target.name) != 'undefined') && (this.target.name != ''))
            {
                //just display the pattern name part of the full node id.
                //this is the name that the user actually enters when creating connections.
                var firstDash = this.target.name.indexOf('-');
                var secondDash = this.target.name.indexOf('-', firstDash + 1);
                var thirdDash = this.target.name.indexOf('-', secondDash + 1);
                paramCont += this.target.name.substring(secondDash + 1, thirdDash);
            }
            paramCont += '</span>';
            paramCont += '<a style="margin-left : 10px;" href="#" onclick="' + this.getJavascriptIdentifier() + '.changeDBConnection();return false;">';
            if (this.target.name == '')
                paramCont += 'Select';
            else
                paramCont += 'Change';
            paramCont += '</a>';
            if (typeof(this.errors.dbconnection) != 'undefined')
                paramCont += '<span class="errorIcon" onmouseover="showTipMessage();" onmouseout="hideTipMessage();" _tipMessage="'+this.errors.dbconnection.getMessage()+'">'+ERROR_TAG_CONTENT_STRING+'</span>';
            paramCont += '</td></tr>';
        }


        for (pr in this.currentParams)
        {
            if ((this.operator != 'SQL') || (this.currentParams[pr].name == 'to_location') || (this.currentParams[pr].name == 'sql_statement'))
                paramCont += this.currentParams[pr].show();
        }

        //if this is a custom logicsheet check if this result has non standard params (even if it is a standard operator name).
        //if so then we wont show any of the target/service information
        var showTargetInfo = true;
        if (this.rule.ruleBase.isCustom)
        {
            for (pr in this.currentParams)
            {
                if (resultOperatorsDoc.selectSingleNode("/hy:rulemaker/hy:results/hy:operator[@name = '"+this.operator+"']/hy:params/hy:param[@name = '"+this.currentParams[pr].name+"']") == null)
                {
                    showTargetInfo = false;
                    break;
                }
            }
        }

        // show result target settings - update method, and action/service for Service calls
        if (showTargetInfo)
        {
            var updateMethods = resultOperatorsDoc.selectNodes("/hy:rulemaker/hy:results/hy:operator[@name = '"+this.operator+"']/hy:targets/hy:target/hy:update");
            if ((updateMethods.length > 0) || (this.operator == 'Service'))
            {
                //var targetHTML = '<table border="0">';
                var targetHTML = "";
                if (updateMethods.length > 0)
                {
                    targetHTML += '<tr><td><label for="rule_' + this.rule.getID() + '_result_' + this.displayID + '_target_update">Merge Option</label></td>';
                    targetHTML += '<td><select id="rule_' + this.rule.getID() + '_result_' + this.displayID + '_target_update" onchange="'+this.getJavascriptIdentifier()+'.target.update = this.value; '+this.rule.getJavascriptIdentifier()+'.setModified();">';

                    for (var i = 0; i < updateMethods.length; ++i)
                    {
                        var um = updateMethods.item(i).firstChild.nodeValue;
                        var dispName = updateMethods.item(i).getAttribute("display_name");
                        targetHTML += '<option value="' + um + '"';
                        if (um == this.target.update)
                        {
                            targetHTML += ' selected="selected"';
                        }
                        targetHTML += '>';
                        if ((dispName != null) && (dispName != ''))
                            targetHTML += dispName;
                        else
                            targetHTML += um;
                        targetHTML += '</option>';
                    }

                    targetHTML += '</select></td></tr>';
                }
                if (this.operator == 'Service')
                {
                    targetHTML += '<tr><td><label for="rule_' + this.rule.getID() + '_result_' + this.displayID + '_target_name">Service</label></td>';
                    targetHTML += '<td><input size="65" type="text" id="rule_' + this.rule.getID() + '_result_' + this.displayID + '_target_name" onchange="'+this.getJavascriptIdentifier()+'.target.name = this.value; '+this.rule.getJavascriptIdentifier()+'.setModified();" value="'+this.target.name+'"/>';

                    if (typeof(this.errors.service) != 'undefined')
                        targetHTML += '<span class="errorIcon" onmouseover="showTipMessage();" onmouseout="hideTipMessage();" _tipMessage="'+this.errors.service.getMessage()+'">'+ERROR_TAG_CONTENT_STRING+'</span>';

                    targetHTML += '</td></tr><tr><td><label for="rule_' + this.rule.getID() + '_result_' + this.displayID + '_target_action">Action</label></td>';
                    targetHTML += '<td><input size="65" type="text" id="rule_' + this.rule.getID() + '_result_' + this.displayID + '_target_action" onchange="'+this.getJavascriptIdentifier()+'.target.action = this.value; '+this.rule.getJavascriptIdentifier()+'.setModified();" value="'+this.target.action+'"/></td></tr>';


                    targetHTML += '<td></td><td><a style="float:left; width : 45%;" href="javascript:invokeServiceCallHelper('+this.getJavascriptIdentifier()+');">Service Call Helper</a>';
                    if (checkTargetAvailable(this.target.name))
                    {
                        targetHTML += '<a style="float:left; width : 45%;" href="javascript:viewTargetRules(\'rule_' + this.rule.getID() + '_result_' + this.displayID + '_target_name\')">View Target Rules</a>';
                    }

                    targetHTML += '</td></tr>';
                }

                //check if the parallel option is available for this result, and if so output the checkbox
                if (resultOperatorsDoc.selectSingleNode("/hy:rulemaker/hy:results/hy:operator[@name = '"+this.operator+"']/hy:targets/hy:target/hy:parallel"))
                {
                    targetHTML += '<tr><td><label for="rule_' + this.rule.getID() + '_result_' + this.displayID + '_parallel">Run this Action in Parallel?</label></td>';
                    targetHTML += '<td><input type="checkbox" id="rule_' + this.rule.getID() + '_result_' + this.displayID + '_parallel" onchange="'+this.getJavascriptIdentifier()+'.target.parallel = this.checked; '+this.rule.getJavascriptIdentifier()+'.setModified();" value="true" ';
                    if (this.target.parallel)
                        targetHTML += 'checked="checked"/>';
                    else
                        targetHTML += '/>';

                    targetHTML += '</td></tr>';
                }

                //targetHTML += '</table>';

                //cont.innerHTML += targetHTML;
                paramCont += targetHTML;

            }
        }

        paramCont += "</table>";
        cont.innerHTML += paramCont;


        document.getElementById("rule_" + this.rule.getID() + "_result_" + this.displayID + "_container").appendChild(cont);

        document.getElementById("rule_"+this.rule.getID()+"_result_header_button_"+this.displayID).className = "collapsible_open";
        this.expanded = true;

        //check if any of the parameters should use the editarea, and if so try and initialise it
        for (pr in this.currentParams)
        {
            if (this.currentParams[pr].useEditArea)
            {
                convertToEditArea('rule_' + this.getRule().getID() + '_result_' + this.getID() + '_param_' + this.currentParams[pr].displayID +'_value', this.currentParams[pr].editAreaType);
            }
        }

        //add in the SQL params
        if (this.operator == 'SQL')
        {
            for (pr in this.currentParams)
            {
                if (this.currentParams[pr].name == 'sql_statement')
                    this.sqlParamValueChanged(this.currentParams[pr].value);
            }
        }

    }
    else
    {
        //check if any of the parameters should use the editarea, and if so try and destroy it
        for (pr in this.currentParams)
        {
            if (this.currentParams[pr].useEditArea)
            {
                try
                {
                    editAreaLoader.delete_instance('rule_' + this.getRule().getID() + '_result_' + this.getID() + '_param_' + this.currentParams[pr].displayID +'_value');
                }
                catch (e)
                {}  //ignore

            }
        }

        existingParams.parentNode.removeChild(existingParams);
        document.getElementById("rule_"+this.rule.getID()+"_result_header_button_"+this.displayID).className = "collapsible_hidden";
        this.expanded = false;
    }
}

/**
 * Called when the value of the operator dropdown for this result is changed.
 * Updates the param details for this result to match the new operator.
 */
function operatorChanged(dropdown)
{
    this.operator = dropdown.value;

    // check for and remove the Please Select option if present
    for (var i = 0; i < dropdown.options.length; ++i)
    {
        if (dropdown.options[i].value == "")
            dropdown.options[i] = null;
    }

    //check if any of the parameters should use the editarea, and if so try and destroy it
    for (pr in this.currentParams)
    {
        if (this.currentParams[pr].useEditArea)
        {
            try
            {
                editAreaLoader.delete_instance('rule_' + this.getRule().getID() + '_result_' + this.getID() + '_param_' + this.currentParams[pr].displayID +'_value');
            }
            catch (e)
            { } //edit are probably didnt exist, so just ignore.
        }
    }

    this.operatorParams = resultOperatorsDoc.selectSingleNode("/hy:rulemaker/hy:results/hy:operator[@name = '"+this.operator+"']/hy:params");
    this.currentParams = new Object();

    //update target info
    opTarget = resultOperatorsDoc.selectSingleNode("/hy:rulemaker/hy:results/hy:operator[@name = '"+this.operator+"']/hy:targets/hy:target");
    this.target.name = opTarget.getAttribute("name");
    opAction = opTarget.selectSingleNode("hy:action");
    if ((opAction != null) && (opAction.firstChild != null))
        this.target.action = opAction.firstChild.nodeValue;
    else
        this.target.action = "";
    opUpdate = opTarget.selectSingleNode("hy:update[1]");
    if ((opUpdate != null) && (opUpdate.firstChild != null))
        this.target.update = opUpdate.firstChild.nodeValue;
    else
        this.target.update = "";

    paramsNL = this.operatorParams.selectNodes("hy:param[not(@sequence = preceding-sibling::hy:param/@sequence)]");

    for (var i = 0; i < paramsNL.length; ++i)
    {
        if (paramsNL.item(i).getAttribute("name") != 'cache')
            this.currentParams[i] = new Param(i, null, this, paramsNL.item(i).getAttribute("sequence"));
    }

    // if the operator is a sql one, ensure the default sql is parsed correctly.
    if (this.operator == 'SQL')
    {
        for (pr in this.currentParams)
        {
            if (this.currentParams[pr].name == 'sql_statement')
                this.sqlParamValueChanged(this.currentParams[pr].value);
        }
    }

    //update hint content
    var hintContainer = document.getElementById('rule_'+this.rule.getID()+'_result_header_'+this.displayID+'_hint');
    hintContainer.setAttribute("_tipMessage", getActionHint(this.operator));

    this.toggleResultVisibility();
    this.toggleResultVisibility();

    this.rule.setModified();
    this.rule.updateRuleSummary();
}

/**
 * Called when the value of the main sql statement param within this SQL result has been changed.
 * This then parses the new sql string to identify all the param values it needs, to display the
 * appropriate entry fields.
 */
function sqlParamValueChanged(sqlString)
{
    if (this.operator == 'SQL')
    {
        var testString = sqlString + ' ';
        var sqlparams = new Array();
        rePattern = /(\$)([\$\w]+)\W/g;
        while (arrMatch = rePattern.exec(testString))
        {
            if (arrMatch[2].indexOf('$') == 0)
            {
                //was actually two $ symbols together (ie escaped) so ignore
            }
            else
            {
                if (!arrayContains(sqlparams, '$' + arrMatch[2]))
                    sqlparams.push('$' + arrMatch[2]);
            }
        }

        var newParams = new Object();

        //push the sql string and to location params across
        for (pr in this.currentParams)
        {
            if (this.currentParams[pr].name == 'sql_statement')
            {
                newParams[0] = this.currentParams[pr];
                newParams[0].displayID = 0;
            }
            else if (this.currentParams[pr].name == 'to_location')
            {
                newParams[1] = this.currentParams[pr];
                newParams[1].displayID = 1;
            }
        }

        paramHTML = '<table>';

        //create all the param values now needed for the sql statement
        for (var i = 0; i < sqlparams.length; ++i)
        {
            var newid = getPropertyCount(newParams);
            var np = new Param(newid, null, this, null);
            np.name = sqlparams[i];
            np.type = "com.hyfinity.xpath.XPath";
            np.value = "/mvc:eForm/mvc:Control/mvc:param_value";
            np.paramUse = 'SQL';
            //check if it existed before, and if so update the value
            for (pr in this.currentParams)
            {
                if (this.currentParams[pr].name == np.name)
                {
                    np.value = this.currentParams[pr].value;
                    np.paramDataType = this.currentParams[pr].paramDataType;
                    break;
                }
            }
            newParams[newid] = np;

            paramHTML += np.show();

        }

        paramHTML += '</table>';


        this.currentParams = newParams;

        var paramCont = document.getElementById('rule_' + this.rule.getID() + '_result_' + this.displayID + '_sql_params');
        if (sqlparams.length == 0)
        {
            if (paramCont != null)
            {
                paramsRow = paramCont.parentNode.parentNode;
                paramsRow.parentNode.removeChild(paramsRow);
            }
        }
        else
        {
            if (paramCont == null)
            {
                resultCont = document.getElementById("rule_" + this.rule.getID() + "_result_" + this.displayID + "_detail");
                if (resultCont != null)
                {
                    var paramTable = resultCont.firstChild;

                    var newRow = paramTable.insertRow(-1);
                    var newTD = document.createElement('td');
                    newTD.className = 'sqlParams';
                    newTD.colSpan = 2;
                    newRow.appendChild(newTD);

                    newTD.innerHTML = '<div class="sqlParamsHeader">SQL Parameters</div><div id="rule_' + this.rule.getID() + '_result_' + this.displayID + '_sql_params"><table>' +
                                            paramHTML + '</table></div>';
                }
            }
            else
            {
                paramCont.innerHTML = paramHTML;
            }
        }
    }
}


/**
 * Uses the service call helper to let the user pick (or create) a DB connection (model node)
 * to use for this sql operator.
 */
function changeDBConnection()
{
    selectedResultObj = this;
    document.serviceCallHelperCaller.new_pattern_collection.value = document.getElementById('proxy_collection').value;
    document.serviceCallHelperCaller.layer_filter.value = document.getElementById('proxy_layer_id').value;
    document.serviceCallHelperCaller.display_mode.value = 'rdbms';
    window.open("", "serviceCallHelperWindow", 'width=900, height=450, toolbar=no, menubar=no, directories=no, scrollbars=yes, resizable=yes, status=yes');
    document.serviceCallHelperCaller.submit();

}

/**
 * Called when the disabled checkbox is ticked for this rule
 * @param checkbox The checkbox indicating the disabled state of the rule
 */
function setResultDisabled(newDisabled, ignoreChange)
{
    this.disabled = newDisabled;
    var resultContainer = document.getElementById('rule_'+this.rule.getID()+'_result_'+this.displayID+'_container')
    if ((this.disabled) && (this.rule.ruleBase.rulebaseSettings.disable))
        resultContainer.className = "collapsible_section possibleFloat marginAdjustment disabled_rule";
    else if (this.disabled)
        resultContainer.className = "collapsible_section possibleFloat marginAdjustment disabled_rule_off";
    else
        resultContainer.className = "collapsible_section possibleFloat marginAdjustment";

    if (typeof(ignoreChange) == 'undefined')
        this.rule.setModified();
}


/**
 * Called when an attempt is made to show the context menu on a rule.
 * This positions our specific context menu, and returns false to stop
 * the default one from being shown.
 */
function showResultContextMenu(e)
{
    //ensure the result specific options are visible
    showComponent(document.getElementById("context_menu_result_options"));

    //store this result as the context menu target if not already set
    if (contextMenuTarget == null)
        contextMenuTarget = this;
}

/**
 * Attaches the given ValidationError object to this result so
 * that it will be displayed the next time this result is reshown.
 * @param error The ValidationError object describing the error
 */
function attachResultError(error)
{
    switch (error.type)
    {
        case ValidationError.INVALID_RESULT_ID :
                    this.errors.name = error;
                    break;
        case ValidationError.DUPLICATE_RESULT_ID    :
                    //invalid result id errors should always override duplicate id errors
                    if ((typeof(this.errors.name) == 'undefined') || (this.errors.name.type != ValidationError.INVALID_RESULT_ID))
                    {
                        this.errors.name = error;
                    }
                    break;
        case ValidationError.MISSING_RESULT_OPERATOR :
                    this.errors.operator = error;
                    break;
        case ValidationError.MISSING_RESULT_SERVICE_TARGET :
                    this.errors.service = error;
                    break;
        case ValidationError.MISSING_RESULT_DB_CONNECTION :
                    this.errors.dbconnection = error;
                    break;
        default :   //not an error associated with a result so ignore.
    }
}

/**
 * Clears all errors stored against this result and any params
 */
function clearResultErrors()
{
    this.errors = new Object();
    for (pr in this.currentParams)
    {
        this.currentParams[pr].clearErrors();
    }
}

/**
 * Checks whether there is an error defined for this result (or its params)
 * @return A boolean value - true if an error is defined, false otherwise.
 */
function hasResultError()
{
    if (getPropertyCount(this.errors) > 0)
        return true;
    else
        return this.hasParamError();
}

/**
 * Checks if a param associated with this result has an error or not.
 * @private
 */
function hasResultParamError()
{
    for (pr in this.currentParams)
    {
        if (this.currentParams[pr].hasError())
            return true;
    }
    return false;
}


/**
 * Returns the rule object that this result belongs to.
 * @return the Rule object for this result.
 */
function getResultRule()
{
    return this.rule;
}

/**
 * Returns the display ID of this result for this instance of the rulebase editor.
 * @return the Rule object for this result.
 */
function getResultID()
{
    return this.displayID;
}

/**
 * Returns the javascript identifier for this result object.
 */
function getResultJavascriptIdentifier()
{
    return this.rule.getResultsIdentifier() + "[" + this.displayID+ "]";
}

/**
 * Returns the javascript identifier for the currentParams object held by this result object.
 */
function getParamsIdentifier()
{
    return this.getJavascriptIdentifier() + ".currentParams";
}


/**
 * creates an XML string to describe this result object
 * @return The XML formatted string that describes this result.
 */
function getResultXML()
{
    var resultXML = '<hy:result xmlns:hy="http://www.hyfinity.com/xengine" id="'+this.name+'" disable="'+this.disabled+'">';

    resultXML += '<hy:operator>' + this.operator + '</hy:operator>';
    resultXML += '<hy:target action="' + this.target.action + '" name="' + this.target.name + '" update="' + this.target.update + '"';

    if (resultOperatorsDoc.selectSingleNode("/hy:rulemaker/hy:results/hy:operator[@name = '"+this.operator+"']/hy:targets/hy:target/hy:parallel"))
    {
        resultXML += ' parallel="' + this.target.parallel + '"';
    }

    if (typeof(this.target.type) != 'undefined')
    {
        resultXML += ' type="' + this.target.type + '"';
    }

    resultXML += '>';
    resultXML += '<hy:params>';

    for (pr in this.currentParams)
    {
        resultXML += this.currentParams[pr].getParamXML();
    }


    resultXML += '</hy:params>';
    resultXML += '</hy:target>';

    resultXML += '</hy:result>';

    return resultXML;
}

/**
 * Returns an XML string detailing the curernt visibility state of this result object
 * @return The XML formated string containing state information.
 */
function getResultStateXML()
{
    return '<hy:result id="'+this.name+'" visible="'+this.expanded+'" xmlns:hy="http://www.hyfinity.com/xengine"/>';
}





//************************Utility Functions************************

/**
 * Returns a string containing the HTML to render a select box with
 * the list of result operators as the set of options
 * @param name The name (and id) to use for the select box
 * @param value The value to select. should be one of the available options
 * @param custom Boolean indicating whether this is a custom logicsheet
 * @param onchange The string to place in the onchange event (optional)
 * @returns the HTML String
 */
function getOperatorsSelectHTML(name, selected, custom, onchange)
{
    if (!custom)
    {
        var selectString = '<select name="' + name + '" id="' + name + '"';
        if (typeof(onchange) != 'undefined')
            selectString += ' onchange="' + onchange + '"';
        selectString += '>';

        var operators = resultOperatorsDoc.selectNodes("/hy:rulemaker/hy:results/hy:operator");

        if (selected=="")
            selectString += "<option value=''>Please Select</option>";

        for (var i = 0; i < operators.length; ++i)
        {
            var opName = operators.item(i).getAttribute("name");
            var dispName = operators.item(i).getAttribute("display_name");
            selectString += '<option value="' + opName + '"';
            if (opName == selected)
                selectString += ' selected="selected"';
            selectString += '>';
            if ((dispName != null) && (dispName != ""))
            {
                selectString += dispName;
            }
            else
            {
                selectString += opName;
            }
            selectString += '</option>';
        }

        selectString += '</select>';

        return selectString;
    }
    else
    {
        //custom logicsheet so dont allow changing of operator
        //return '<span style="border:thin solid red; height:21px;vertical-align:middle;">'+selected+'</span>';
        return selected;
    }


}

/**
 * Checks to see if the given target (node) is available to view the rules for.
 * This currently checks whether the target node is in this project or not.
 * TODO: Needs to check if the target is a proxy node somehow....
 * @param target The four part id of the target node.
 */
function checkTargetAvailable(target)
{
    var array = target.split("-");
    if ((projectDetails.product == array[0]) &&
        (projectDetails.project == array[1]))
    {
        return true;
    }
    else
    {
        return false;
    }

}

/**
 * Returns a string that can be used as a hint for an action with the specified operator.
 * The String returned will be properly escaped to be placed in an attribute value.
 */
function getEscapedActionHint(operator)
{

    var hint = getActionHint(operator);
    hint = hint.replace(/</gi , "&lt;");
    hint = hint.replace(/>/gi , "&gt;");
    hint = hint.replace(/"/gi , "&quot;");

    return hint;
}

/**
 * Returns the name to use for displaying the given operator.
 * @param operator The name of the operator to get the display name for.
 * @return A string containing the name to use to duisplay this operator.
 */
function getOperatorDisplayName(operator)
{
    var op = resultOperatorsDoc.selectSingleNode("/hy:rulemaker/hy:results/hy:operator[@name ='" + operator + "']");
    if (op == null)
    {
        return operator
    }
    else
    {
        var disp = op.getAttribute("display_name");
        if (disp == null)
        {
            return operator;
        }
        else
        {
            return disp;
        }
    }
}


/**
 * Returns a string that can be used as a hint for an action with the specified operator.
 */
function getActionHint(operator)
{
    var hint = "Each Action defines one operation that should be performed when this rule gets fired.<br /><br />";
    hint += "The Name is used to identify each action within the rule, and the drop-down selects the type of action to be performed.";

    hint += "<br /><br />";

    if ((typeof(operator) == 'undefined') || (operator == ''))
    {
        hint += "Please select an action type for more information.";
        return hint;
    }

    hint += "<span class=\"hint_heading\">"+getOperatorDisplayName(operator)+"</span>";

    hint += "<br /><br />";

    switch (operator)
    {
        case 'Write'    :   hint += "This action writes a value into a specified location in the XML data.<br /><br />The first parameter specifies the location to write to, and the second parameter either contains the String value to write, or alternatively specifies another location in the data to get the value from."; break;
        case 'Save'     :   hint += "This action is used to serialize all or part of the XML data in the factbase out to a file.<br /><br />The first parameter specifies which part of the data should be saved out.  The second parameter either specifies the resulting filename directly, or defines the location in the data of the filename to use.<br /><br />The filename can either be absolute, or relative, in which case it is assumed to be relative to the deployed runtime location of this node."; break;
        case 'Return'   :   hint += "This action can be used to immediately stop processing in this node.<br /><br />When this action is encountered, the current XML data specified by the Rulebase Return setting is returned to the calling node/application.<br /><br />No parameters are required for this Action."; break;
        case 'Delete'   :   hint += "This action is used to remove a section of data from the factbase.<br/><br/>It requires one parameter which is an XPath to the XML node to delete.<br/>Any children of this node will also be deleted."; break;
        case 'Copy'     :   hint += "The Copy action is used to copy a fragment of XML data from one location within the factbase to another.<br /><br />This requires two parameters.  The first specifies the location of the data to copy, the second specifies the location in which the copy should be placed.<br /><br />If the update method is 'Append' then the copied data will be placed as a child of the target node, where as if it is 'Replace' the copied data will replace the target node."; break;
        case 'Parse'    :   hint += "The Insert action loads an XML node into the factbase to supplement the initial incoming XML message. <br /><br />This action requires two parameters. The first defines the location in the factbase to insert the new data into, and the second specifies the XML to parse in.<br />This XML can be loaded from an external XML file (whose filename is either specified directly or found by using an XPath into the data), or it can be described as a fragment of XML within the logicsheet.<br /><br />If the update method is 'Append' then the parsed data will be placed as a child of the insert location node, where as if it is 'Replace' the input data will replace this location node."; break;
        case 'AddAttribute': hint += "This action adds an new attribute to a specified node in the XML data.<br /><br />This requires three parameters.<br/>The first specifies the element the attribute should be added to, the second defines the name of the new attribute, and the third provides the initial value to give to the new attribute."; break;
        case 'Cache'    :   hint += "This action is used to store a fragment of XML data in either the users 'session' or the 'application' cache.<br /><br />Three parameters are required.<br />The first specifies the location in the factbase of the data to cache.  The second parameter provides the key or id under which the data should be cached, and the third indicates whether the data should be stored in the 'session' or 'application' cache."; break;
        case 'ClearCache':  hint += "This is used to remove information stored in the cache.<br /><br />The first parameter specifies the id under which the data is stored, and the second parameter specifies whether the data should be removed from the 'session' or 'application' cache."; break;
        case 'Retrieve' :   hint += "This action allows a cached fragment of XML data to be retrieved from the cache.<br /><br />Three parameters are required by this action.<br />The location under which to append the retrieved data, the id value under which the data has been cached, and the type of cache the data is stored in, either 'session' or 'application'."; break;
        case 'Validate' :   hint += "This action validates a specified section of the factbase against a defined W3C Schema definition.<br /><br />There are three parameters required for this action.<br />The first specifies the schema file to use for the validation (either entered directly or found using an XPath into the data).<br />The second parameter indicates the portion of the factbase to validate against the schema.<br />The final parameter defines the location under which to place details of any errors found during the validation process."; break;
        case 'Transform':   hint += "This action performs an XSL transformation on the contents of the factbase using an appropriate XSL.<br /><br />The first parameter defines the XSLT stylesheet to use for the transformation.  This can either be specified directly, or defined as an XPath into the XML data to retrieve the filename.<br /><br />The second parameter indicates the section of the factbase that should be transformed, and the third parameter indicates where to place the results of the transform.<br />The output data can either be placed back into the factbase, or written directly out to a file.<br /><br />When writting back to the factbase, if the update method is 'Append' then the resulting data will be placed as a child of the target node, where as if it is 'Replace' the transform output will replace this target node."; break;
        case 'Service'  :   hint += "This is the main orchestration operation.<br /> A Service call can be made to any other node within XPlatform. The target nodes may be native XPlatform nodes or proxies to remote web services.<br /> The Service operation always assumes it is calling a named native XPlatform node. The physical manifestation of this node can be specified during deployment of the application.<br /><br />This action requires two parameters, the section of the factbase that should be sent as the request to the target node, and the location in which to put the response.<br /><br />In addition, this action requires the details of the target XPlatform node to be provided.  This can be done easily using the 'Service Call Helper' option.<br /><br />If the update method is 'Append' then the received XML response data will be placed as a child of the specified target element, where as if it is 'Replace' the received data will replace this element."; break;
        case 'Log'      :   hint += "This enables the logging of XML information to the underlying XPlatform logging system.<br /><br />You need to specify the section of the factbase that should be logged, the type of log file that should be logged to ('platform', 'developer', or 'administration'), and for platform logs the level at which the log should be made."; break;
        case 'Compare'  :   hint += "This action is used to check for differences between two sets of XML data.<br /><br />Both the source and target XML (first and second parameters) can be taken from the factbase or from external files.<br />The third parameter specifies the location to place the result information, and the final parameter controls whether detailed output is required ('true' or 'false')."; break;
        case 'XMLStringConverter' : hint += "This action is used to convert between XML formatted strings and actual XML data.<br /><br />This is often needed when communicating with web services that return XML formatted strings instead of 'real' XML structures to try and provide flexibility.<br /><br />This action provides two 'modes', either converting XML strings to XML, or the other way round.  In each case you need to specify a location XPath to indicate which element should be converted to a string, or which element's string content should be converted into XML."; break;
        case 'DirectoryCopy' : hint += "This action is used to copy a directory (or individual file) from one location on the file system to another.<br /><br />The first parameter sets the source file or directory to copy, and the second sets the directory it should be copied into.<br />Each value can be entered directly or determined by evaluating an XPath against the runtime data.<br /><br />Any relative path locations will be determined based on the deployed location of this node."; break;
        case 'DeleteFile' : hint += "This action is used to delete a file (or directory) from the file system.<br /><br />It requires a single parameter which specifies the file to delete.  This can either be specified at design time, or found by evaluating an XPath against the current factbase at runtime.<br /><br />Relative file locations entered are relative to the deployed location of this node."; break;
        case 'FileInformation' : hint += "The File Information action can be used to obtain information about files or directories on the file system.<br /><br />This action requires two parameters.  The first specifies the file or directory to retrieve the information about, which can either be entered directly or obtained dynamically at runtime.<br />The information obtained by this action is returned as an XML fragment which is inserted into the factbase under the location specified by the second parameter.<br /><br />Examples of the information returned by this action include whether or not the file is hidden, whether it is writeable, and the last modified timestamp.  In addition, if the first parameter points to a directory, a listing of the contents of the directory will also be returned.<br /><br />Please consult the documentation for more details on the format of this information fragment."; break;
        case 'Move'     :   hint += "The Move action is used to move a fragment of XML data from one location within the factbase to another.<br /><br />This requires two parameters.  The first specifies the location of the data that is being moved, the second specifies the location it should be moved to.<br /><br />If the update method is 'Append' then the data will be placed as a child of the target node, where as if it is 'Replace' the moved data will replace the target node."; break;
        case 'SQL'      :   hint += "The SQL action provides access to relational databases.  This provides the ability to define parameterised SQL queries and dynamically populate the parameters from incoming data. The resulting information from the queries is placed into the factbase for further processing.<br /><br />The first parameter sets which database connection the entered statements should be evaluated against.  Please use the 'Select' or 'Change' button as appropriate to set this.<br /><br />The actual SQL that you would like to execute needs to be entered into the second parameter.  Multiple statements should be separated using the ';' character.  Any dynamic values needed in the SQL should be represented using the '$param' syntax.<br /><br />The final required parameter contains the XPath indicating where the results from evaluating the SQL query should be placed.<br /><br />If you have made use of any dynamic parameters in your SQL statement, then you will also need to specify the XPaths that should be used to retrieve these values, and the appropriate data types of the values."; break;
        case 'Rename'   :   hint += "The Rename action is used to rename elements or attributes within the factbase.<br /><br />The first parameter specifies the location of the element or attribute that should be renamed, the second provides its new name, and the third sets the namespace in which it should be placed.<br /><br />Both the new name and new namespace values can be entered at design time, or determined dynamically from the runtime data."; break;


    }

    return hint;

}




var selectedResultObj;

function invokeServiceCallHelper(object)
{
    selectedResultObj = object;
    document.serviceCallHelperCaller.layer_filter.value = '';
    document.serviceCallHelperCaller.new_pattern_collection.value = '';
    document.serviceCallHelperCaller.display_mode.value = 'normal';
    window.open("", "serviceCallHelperWindow", 'width=900, height=400, toolbar=no, menubar=no, directories=no, scrollbars=yes, resizable=yes, status=yes');
    document.serviceCallHelperCaller.submit();
}

function serviceCallHelper(target, pattern, node, action)
{
    delete selectedResultObj.target.type;
    selectedResultObj.target.name = target;
    selectedResultObj.target.action = action;
    selectedResultObj.rule.setModified();
    selectedResultObj.toggleResultVisibility();
    selectedResultObj.toggleResultVisibility();
    document.rulemaker.xml_processing.value += '<add_splice><xfact:target_node xmlns:xfact="http://www.hyfinity.com/xfactory"><xfact:pattern_id>' + pattern + '</xfact:pattern_id><xfact:node_id>' + node + '</xfact:node_id><xfact:action>' + action + '</xfact:action></xfact:target_node></add_splice>';
}

function viewTargetRules(targetID)
{
    var target = document.getElementById(targetID);
    var array = target.value.split("-");
    //rulemakerTarget.target = target.value;
    document.rulemakerTarget.node_id.value = array[3];
    document.rulemakerTarget.pattern_id.value = array[2];
    //rulemakerTarget.submit();
    WinManager.resolveFormTargetStart(document.rulemakerTarget, 'RuleMaker', document.serviceCallHelperCaller.project_name.value,
                        document.rulemakerTarget.pattern_id.value, document.rulemakerTarget.node_id.value);
}